/*
 * Part of the SlogSig project, an attempt to provide a type-safe
 * signals/slots mechanism for the C++ language.
 *
 * Copyright (C) 2003 Yves Bailly <kafka_AT_kafka-fr_DOT_net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

#ifndef __SLOTSIG_COMMON_H__
#define __SLOTSIG_COMMON_H__ 1

#include <string>
#include <set>
#include <algorithm>

namespace nsSlotSig {

class clSenderBase
{
  public:
    clSenderBase() :
      m_SenderClassName(""),
      m_SenderName("")
    { }

    clSenderBase(const std::string& sender_class_name,
                 const std::string& sender_name) :
      m_SenderClassName(sender_class_name),
      m_SenderName(sender_name)
    { }

    const std::string& senderClassName() const
    { return m_SenderClassName ; }

    const std::string& senderName() const
    { return m_SenderName ; }

  protected:
    void setSenderClassName (const std::string& str)
    { m_SenderClassName = str ; }

    void setSenderName (const std::string& str)
    { m_SenderName = str ; }

  private:
    std::string m_SenderClassName ;
    std::string m_SenderName ;
} ; // class clSenderBase

template <class _OTHER_TYPE_> class clSlotSigBase ;

template <class _SLOT_SIG_>
class clSlotSigRemover
{
  public:
    clSlotSigRemover(clSlotSigBase<_SLOT_SIG_>* slot_sig) :
      m_SlotSig(slot_sig)
    { }

    void operator() (_SLOT_SIG_* sig_slot)
    {
      sig_slot->remove(m_SlotSig) ;
    }

  private:
    clSlotSigBase<_SLOT_SIG_>* m_SlotSig ;
} ;

} // namespace nsSlotSig

#endif // __SLOTSIG_COMMON_H__
