/*
 * Part of the SlogSig project, an attempt to provide a type-safe
 * signals/slots mechanism for the C++ language.
 *
 * Copyright (C) 2003 Yves Bailly <kafka_AT_kafka-fr_DOT_net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

#include "slotsig_defs.h"

namespace nsSlotSig
{

template <typename _RETURN_TYPE_
          __COMMA__
          __TEMPLATE_ARGS__>
class __CL_SLOTSIG_BASE_NAME__
{
  public:
    typedef __CL_SLOTSIG_BASE_NAME__<_RETURN_TYPE_
                                     __COMMA__
                                     __TEMPLATE_PARAMS__> clSlotSigBase ;

    __CL_SLOTSIG_BASE_NAME__ (clSenderBase* send = 0) :
      m_SlotSigSet(),
      m_Sender(send)
    { }

    virtual ~__CL_SLOTSIG_BASE_NAME__()
    {
      disconnectAll() ;
    }

    virtual void connect (clSlotSigBase* slot_sig)
    {
      if ( slot_sig != 0 )
      {
        add(slot_sig) ;
        slot_sig->add(this) ;
      }
    }

    virtual bool disconnect (clSlotSigBase* slot_sig)
    {
      if ( slot_sig != 0 )
      {
        if ( remove(slot_sig) )
        {
          slot_sig->remove(this) ;
          return true ;
        }
        else
          return false ;
      }
      else
        return false ;
    }

    void disconnectAll()
    {
      register typename std::set<clSlotSigBase*>::const_iterator iter = m_SlotSigSet.begin() ;
      const typename std::set<clSlotSigBase*>::const_iterator end = m_SlotSigSet.end() ;
      while ( iter != end )
        (*(iter++))->remove(this) ;
      m_SlotSigSet.clear() ;
    }

    clSenderBase* sender()
    { return m_Sender ; }
    const clSenderBase* sender() const
    { return m_Sender ; }

    const std::set<clSlotSigBase*>& slotSigSet() const
    { return m_SlotSigSet ; }

    _RETURN_TYPE_ operator() (__METHOD_ARGS__)
    { return _RETURN_TYPE_() ; }

    _RETURN_TYPE_ operator() (clSenderBase* send __COMMA__ __METHOD_ARGS__)
    { return _RETURN_TYPE_() ; }

  protected:
    void add (clSlotSigBase* slotsig)
    { m_SlotSigSet.insert(slotsig) ; }

    bool remove (clSlotSigBase* slotsig)
    { return m_SlotSigSet.erase(slotsig) ; }

    void setSender (clSenderBase* send)
    { m_Sender = send ; }

  private:
    std::set<clSlotSigBase*> m_SlotSigSet ;
    clSenderBase* m_Sender ;
} ;

template <typename _RETURN_TYPE_
          __COMMA__
          __TEMPLATE_ARGS__>
class __CL_SIG_NAME__ ;

template <typename _RETURN_TYPE_
          __COMMA__
          __TEMPLATE_ARGS__>
class __CL_SLOT_NAME__ ;

#include "slotsig_gen_slot.cpp"
#include "slotsig_gen_slot_inclass.cpp"
#include "slotsig_gen_sig.cpp"

} // namespace nsSlotSig

