/*
 * Part of the SlogSig project, an attempt to provide a type-safe
 * signals/slots mechanism for the C++ language.
 *
 * Copyright (C) 2003 Yves Bailly <kafka_AT_kafka-fr_DOT_net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

template <typename _RETURN_TYPE_
          __COMMA__
          __TEMPLATE_ARGS__>
class __CL_SIG_NAME__ : public __CL_SLOTSIG_BASE_NAME__<_RETURN_TYPE_
                                                        __COMMA__
                                                        __TEMPLATE_PARAMS__>
{
  friend class __CL_SLOT_NAME__<_RETURN_TYPE_
                                __COMMA__
                                __TEMPLATE_PARAMS__> ;
  public:
    typedef __CL_SLOT_NAME__<_RETURN_TYPE_
                             __COMMA__
                             __TEMPLATE_PARAMS__> clSlot ;
    typedef std::set<clSlot*> clSlotSet ;
    typedef __CL_SLOTSIG_BASE_NAME__<_RETURN_TYPE_
                                     __COMMA__
                                     __TEMPLATE_PARAMS__> clSlotSigBase ;

    typedef _RETURN_TYPE_ (*FuncType_t) (__METHOD_ARGS__) ;
    typedef _RETURN_TYPE_ (*FuncTypeSender_t) (clSenderBase* sender
                                               __COMMA__
                                               __METHOD_ARGS__) ;
    typedef std::set<FuncType_t>        clFuncSet ;
    typedef std::set<FuncTypeSender_t>  clFuncSenderSet ;

    __CL_SIG_NAME__(clSenderBase* send = 0) :
      clSlotSigBase(send),
      m_FuncSet(),
      m_FuncSenderSet()
    { }

    virtual ~__CL_SIG_NAME__()
    {
      m_FuncSet.clear() ;
      m_FuncSenderSet.clear() ;
    }

    template <typename _IN_CLASS_>
    class clSlotInClass : public __CL_SLOT_INCLASS_NAME__<_RETURN_TYPE_,
                                                          _IN_CLASS_
                                                          __COMMA__
                                                          __TEMPLATE_PARAMS__>
    { } ;

    virtual void connect (clSlot* slot_sig)
    { clSlotSigBase::connect(slot_sig) ; }

    virtual void connect    (FuncType_t  func)
    {
      if ( func != 0 )
        m_FuncSet.insert(func) ;
    }

    virtual void connect    (FuncTypeSender_t funcs)
    {
      if ( funcs != 0 )
        m_FuncSenderSet.insert(funcs) ;
    }

    virtual bool disconnect (clSlot* slot_sig)
    { return clSlotSigBase::disconnect(slot_sig) ; }

    virtual void disconnect (FuncType_t  func)
    {
      if ( func != 0 )
        m_FuncSet.erase(func) ;
    }

    virtual void disconnect (FuncTypeSender_t funcs)
    {
      if ( funcs != 0 )
        m_FuncSenderSet.erase(funcs) ;
    }

    virtual void disconnectAll()
    {
      clSlotSigBase::disconnectAll() ;
      m_FuncSet.clear() ;
      m_FuncSenderSet.clear() ;
    }

    virtual void emit(__METHOD_ARGS__)
    {
      emit (sender() __COMMA__ __METHOD_PARAMS__) ;
    }

    virtual void emit(clSenderBase* sender __COMMA__ __METHOD_ARGS__)
    {
      {
        register typename std::set<clSlotSigBase*>::const_iterator slot_iter = slotSigSet().begin() ;
        const typename std::set<clSlotSigBase*>::const_iterator end_iter = slotSigSet().end() ;
        clSlot* slot ;
        while ( slot_iter != end_iter )
        {
          slot = dynamic_cast<clSlot*>(*slot_iter) ;
          slot->setSender(sender) ;
          slot->operator() (__METHOD_PARAMS__) ;
          ++slot_iter ;
        }
      }
      {
        register typename clFuncSet::const_iterator func_iter = m_FuncSet.begin() ;
        const typename clFuncSet::const_iterator end_iter = m_FuncSet.end() ;
        while ( func_iter != end_iter )
          (*(func_iter++))(__METHOD_PARAMS__) ;
      }
      {
        register typename clFuncSenderSet::const_iterator func_sender_iter = m_FuncSenderSet.begin() ;
        const typename clFuncSenderSet::const_iterator end_sender_iter = m_FuncSenderSet.end() ;
        while ( func_sender_iter != end_sender_iter )
          (*(func_sender_iter++))(sender __COMMA__ __METHOD_PARAMS__) ;
      }
    }

    template <typename _ACCUMULATOR_TYPE_>
    void emit(_ACCUMULATOR_TYPE_& accum __COMMA__ __METHOD_ARGS__)
    {
      emit (accum, sender() __COMMA__ __METHOD_PARAMS__) ;
    }

    template <typename _ACCUMULATOR_TYPE_>
    void emit(_ACCUMULATOR_TYPE_& accum, clSenderBase* sender __COMMA__ __METHOD_ARGS__)
    {
      {
        register typename std::set<clSlotSigBase*>::const_iterator slot_iter = slotSigSet().begin() ;
        const typename std::set<clSlotSigBase*>::const_iterator end_iter = slotSigSet().end() ;
        clSlot* slot ;
        while ( slot_iter != end_iter )
        {
          slot = dynamic_cast<clSlot*>(*slot_iter) ;
          slot->setSender(sender) ;
          accum(slot->operator() (__METHOD_PARAMS__)) ;
          ++slot_iter ;
        }
      }
      {
        register typename clFuncSet::const_iterator func_iter = m_FuncSet.begin() ;
        const typename clFuncSet::const_iterator end_iter = m_FuncSet.end() ;
        while ( func_iter != end_iter )
          accum((*(func_iter++))(__METHOD_PARAMS__)) ;
      }
      {
        register typename clFuncSenderSet::const_iterator func_sender_iter = m_FuncSenderSet.begin() ;
        const typename clFuncSenderSet::const_iterator end_sender_iter = m_FuncSenderSet.end() ;
        while ( func_sender_iter != end_sender_iter )
          accum((*(func_sender_iter++))(sender __COMMA__ __METHOD_PARAMS__)) ;
      }
    }

    virtual void operator() (__METHOD_ARGS__)
    { emit(__METHOD_PARAMS__) ; }

    virtual void operator() (clSenderBase* send __COMMA__ __METHOD_ARGS__)
    { emit(send __COMMA__ __METHOD_PARAMS__) ; }

    const clFuncSet&        funcSet() const
    { return m_FuncSet ; }

    const clFuncSenderSet&  funcSenderSet() const
    { return m_FuncSenderSer ; }

  private:
    clFuncSet       m_FuncSet ;
    clFuncSenderSet m_FuncSenderSet ;
} ; // class clSig0
